@@ -10,7 +10,7 @@ module ActionView
       # relate to the specified Active Record object. Usage example:
       #
       #    <%= div_for(@person, :class => "foo") do %>
-      #       <%=h @person.name %>
+      #       <%= @person.name %>
       #    <% end %>
       #
       # produces:
@@ -25,8 +25,8 @@ module ActionView
       # that relate to the specified Active Record object. For example:
       #
       #    <%= content_tag_for(:tr, @person) do %>
-      #      <td><%=h @person.first_name %></td>
-      #      <td><%=h @person.last_name %></td>
+      #      <td><%= @person.first_name %></td>
+      #      <td><%= @person.last_name %></td>
       #    <% end %>
       #
       # would produce the following HTML (assuming @person is an instance of
@@ -52,9 +52,9 @@ module ActionView
       #
       #    <li id="person_123" class="person bar">...
       #
-      def content_tag_for(tag_name, record, *args, &block)
-        prefix  = args.first.is_a?(Hash) ? nil : args.shift
-        options = args.extract_options!
+      def content_tag_for(tag_name, record, prefix = nil, options = nil, &block)
+        options, prefix = prefix, nil if prefix.is_a?(Hash)
+        options ||= {}
         options.merge!({ :class => "#{dom_class(record, prefix)} #{options[:class]}".strip, :id => dom_id(record, prefix) })
         content_tag(tag_name, options, &block)
       end
