@@ -1,15 +1,21 @@
-require 'action_view/erb/util'
+require 'active_support/core_ext/object/blank'
+require 'active_support/core_ext/string/output_safety'
 require 'set'
 
 module ActionView
+  # = Action View Tag Helpers
   module Helpers #:nodoc:
     # Provides methods to generate HTML tags programmatically when you can't use
     # a Builder. By default, they output XHTML compliant tags.
     module TagHelper
-      include ERB::Util
+      extend ActiveSupport::Concern
+      include CaptureHelper
 
-      BOOLEAN_ATTRIBUTES = %w(disabled readonly multiple checked).to_set
-      BOOLEAN_ATTRIBUTES.merge(BOOLEAN_ATTRIBUTES.map(&:to_sym))
+      BOOLEAN_ATTRIBUTES = %w(disabled readonly multiple checked autobuffer
+                           autoplay controls loop selected hidden scoped async
+                           defer reversed ismap seemless muted required
+                           autofocus novalidate formnovalidate open pubdate).to_set
+      BOOLEAN_ATTRIBUTES.merge(BOOLEAN_ATTRIBUTES.map {|attribute| attribute.to_sym })
 
       # Returns an empty HTML tag of type +name+ which by default is XHTML
       # compliant. Set +open+ to true to create an open tag compatible
@@ -18,9 +24,21 @@ module ActionView
       # escaping.
       #
       # ==== Options
-      # The +options+ hash is used with attributes with no value like (<tt>disabled</tt> and
-      # <tt>readonly</tt>), which you can give a value of true in the +options+ hash. You can use
-      # symbols or strings for the attribute names.
+      # You can use symbols or strings for the attribute names.
+      #
+      # Use +true+ with boolean attributes that can render with no value, like
+      # +disabled+ and +readonly+.
+      #
+      # HTML5 <tt>data-*</tt> attributes can be set with a single +data+ key
+      # pointing to a hash of sub-attributes.
+      #
+      # To play nicely with JavaScript conventions sub-attributes are dasherized.
+      # For example, a key +user_id+ would render as <tt>data-user-id</tt> and
+      # thus accessed as <tt>dataset.userId</tt>.
+      #
+      # Values are encoded to JSON, with the exception of strings and symbols.
+      # This may come in handy when using jQuery's HTML5-aware <tt>.data()<tt>
+      # from 1.4.3.
       #
       # ==== Examples
       #   tag("br")
@@ -29,16 +47,19 @@ module ActionView
       #   tag("br", nil, true)
       #   # => <br>
       #
-      #   tag("input", { :type => 'text', :disabled => true })
+      #   tag("input", :type => 'text', :disabled => true)
       #   # => <input type="text" disabled="disabled" />
       #
-      #   tag("img", { :src => "open & shut.png" })
+      #   tag("img", :src => "open & shut.png")
       #   # => <img src="open &amp; shut.png" />
       #
-      #   tag("img", { :src => "open &amp; shut.png" }, false, false)
+      #   tag("img", {:src => "open &amp; shut.png"}, false, false)
       #   # => <img src="open &amp; shut.png" />
+      #
+      #   tag("div", :data => {:name => 'Stephen', :city_state => %w(Chicago IL)})
+      #   # => <div data-name="Stephen" data-city-state="[&quot;Chicago&quot;,&quot;IL&quot;]" />
       def tag(name, options = nil, open = false, escape = true)
-        "<#{name}#{tag_options(options, escape) if options}#{open ? ">" : " />"}"
+        "<#{name}#{tag_options(options, escape) if options}#{open ? ">" : " />"}".html_safe
       end
 
       # Returns an HTML block tag of type +name+ surrounding the +content+. Add
@@ -60,20 +81,14 @@ module ActionView
       #   content_tag("select", options, :multiple => true)
       #    # => <select multiple="multiple">...options...</select>
       #
-      #   <% content_tag :div, :class => "strong" do -%>
+      #   <%= content_tag :div, :class => "strong" do -%>
       #     Hello world!
       #   <% end -%>
       #    # => <div class="strong">Hello world!</div>
       def content_tag(name, content_or_options_with_block = nil, options = nil, escape = true, &block)
         if block_given?
           options = content_or_options_with_block if content_or_options_with_block.is_a?(Hash)
-          content_tag = content_tag_string(name, capture(&block), options, escape)
-
-          if block_called_from_erb?(block)
-            concat(content_tag)
-          else
-            content_tag
-          end
+          content_tag_string(name, capture(&block), options, escape)
         else
           content_tag_string(name, content_or_options_with_block, options, escape)
         end
@@ -91,7 +106,7 @@ module ActionView
       #   cdata_section(File.read("hello_world.txt"))
       #   # => <![CDATA[<hello from a text file]]>
       def cdata_section(content)
-        "<![CDATA[#{content}]]>"
+        "<![CDATA[#{content}]]>".html_safe
       end
 
       # Returns an escaped version of +html+ without affecting existing escaped entities.
@@ -107,42 +122,33 @@ module ActionView
       end
 
       private
-        BLOCK_CALLED_FROM_ERB = 'defined? __in_erb_template'
-
-        if RUBY_VERSION < '1.9.0'
-          # Check whether we're called from an erb template.
-          # We'd return a string in any other case, but erb <%= ... %>
-          # can't take an <% end %> later on, so we have to use <% ... %>
-          # and implicitly concat.
-          def block_called_from_erb?(block)
-            block && eval(BLOCK_CALLED_FROM_ERB, block)
-          end
-        else
-          def block_called_from_erb?(block)
-            block && eval(BLOCK_CALLED_FROM_ERB, block.binding)
-          end
-        end
 
         def content_tag_string(name, content, options, escape = true)
           tag_options = tag_options(options, escape) if options
-          "<#{name}#{tag_options}>#{content}</#{name}>"
+          "<#{name}#{tag_options}>#{escape ? ERB::Util.h(content) : content}</#{name}>".html_safe
         end
 
         def tag_options(options, escape = true)
           unless options.blank?
             attrs = []
-            if escape
-              options.each_pair do |key, value|
-                if BOOLEAN_ATTRIBUTES.include?(key)
-                  attrs << %(#{key}="#{key}") if value
-                else
-                  attrs << %(#{key}="#{escape_once(value)}") if !value.nil?
+            options.each_pair do |key, value|
+              if key.to_s == 'data' && value.is_a?(Hash)
+                value.each do |k, v|
+                  if !v.is_a?(String) && !v.is_a?(Symbol)
+                    v = v.to_json
+                  end
+                  v = ERB::Util.html_escape(v) if escape
+                  attrs << %(data-#{k.to_s.dasherize}="#{v}")
                 end
+              elsif BOOLEAN_ATTRIBUTES.include?(key)
+                attrs << %(#{key}="#{key}") if value
+              elsif !value.nil?
+                final_value = value.is_a?(Array) ? value.join(" ") : value
+                final_value = ERB::Util.html_escape(final_value) if escape
+                attrs << %(#{key}="#{final_value}")
               end
-            else
-              attrs = options.map { |key, value| %(#{key}="#{value}") }
             end
-            " #{attrs.sort * ' '}" unless attrs.empty?
+            " #{attrs.sort * ' '}".html_safe unless attrs.empty?
           end
         end
     end
