@@ -1,6 +1,7 @@
 require 'action_view/helpers/javascript_helper'
 require 'active_support/core_ext/array/access'
 require 'active_support/core_ext/hash/keys'
+require 'active_support/core_ext/string/output_safety'
 require 'action_dispatch'
 
 module ActionView
@@ -22,6 +23,10 @@ module ActionView
       include ActionDispatch::Routing::UrlFor
       include TagHelper
 
+      def _routes_context
+        controller
+      end
+
       # Need to map default url options to controller one.
       # def default_url_options(*args) #:nodoc:
       #   controller.send(:default_url_options, *args)
@@ -50,9 +55,9 @@ module ActionView
       #
       # ==== Relying on named routes
       #
-      # If you instead of a hash pass a record (like an Active Record or Active Resource) as the options parameter,
-      # you'll trigger the named route for that record. The lookup will happen on the name of the class. So passing
-      # a Workshop object will attempt to use the +workshop_path+ route. If you have a nested route, such as
+      # Passing a record (like an Active Record or Active Resource) instead of a Hash as the options parameter will
+      # trigger the named route for that record. The lookup will happen on the name of the class. So passing a
+      # Workshop object will attempt to use the +workshop_path+ route. If you have a nested route, such as
       # +admin_workshop_path+ you'll have to call that explicitly (it's impossible for +url_for+ to guess that route).
       #
       # ==== Examples
@@ -63,7 +68,7 @@ module ActionView
       #   # => /books/find
       #
       #   <%= url_for(:action => 'login', :controller => 'members', :only_path => false, :protocol => 'https') %>
-      #   # => https://www.railsapplication.com/members/login/
+      #   # => https://www.example.com/members/login/
       #
       #   <%= url_for(:action => 'play', :anchor => 'player') %>
       #   # => /messages/play/#player
@@ -76,9 +81,12 @@ module ActionView
       #   # => /workshops
       #
       #   <%= url_for(@workshop) %>
-      #   # calls @workshop.to_s
+      #   # calls @workshop.to_param which by default returns the id
       #   # => /workshops/5
       #
+      #   # to_param can be re-defined in a model to provide different URL names:
+      #   # => /workshops/1-workshop-name
+      #
       #   <%= url_for("http://www.example.com") %>
       #   # => http://www.example.com
       #
@@ -91,7 +99,7 @@ module ActionView
       #   # => javascript:history.back()
       def url_for(options = {})
         options ||= {}
-        url = case options
+        case options
         when String
           options
         when Hash
@@ -102,17 +110,15 @@ module ActionView
         else
           polymorphic_path(options)
         end
-
-        url
       end
 
-      # Creates a link tag of the given +name+ using a URL created by the set
-      # of +options+. See the valid options in the documentation for
-      # +url_for+. It's also possible to pass a string instead
-      # of an options hash to get a link tag that uses the value of the string as the
-      # href for the link, or use <tt>:back</tt> to link to the referrer - a JavaScript back
-      # link will be used in place of a referrer if none exists. If +nil+ is passed as
-      # a name, the link itself will become the name.
+      # Creates a link tag of the given +name+ using a URL created by the set of +options+.
+      # See the valid options in the documentation for +url_for+. It's also possible to
+      # pass a String instead of an options hash, which generates a link tag that uses the
+      # value of the String as the href for the link. Using a <tt>:back</tt> Symbol instead
+      # of an options hash will generate a link to the referrer (a JavaScript back link
+      # will be used in place of a referrer if none exists). If +nil+ is passed as the name
+      # the value of the link itself will become the name.
       #
       # ==== Signatures
       #
@@ -180,7 +186,7 @@ module ActionView
       #   link_to "Profiles", :controller => "profiles"
       #   # => <a href="/profiles">Profiles</a>
       #
-      # You can use a block as well if your link target is hard to fit into the name parameter. ERb example:
+      # You can use a block as well if your link target is hard to fit into the name parameter. ERB example:
       #
       #   <%= link_to(@profile) do %>
       #     <strong><%= @profile.name %></strong> -- <span>Check it out!</span>
@@ -238,8 +244,8 @@ module ActionView
           href = html_options['href']
           tag_options = tag_options(html_options)
 
-          href_attr = "href=\"#{html_escape(url)}\"" unless href
-          "<a #{href_attr}#{tag_options}>#{html_escape(name || url)}</a>".html_safe
+          href_attr = "href=\"#{ERB::Util.html_escape(url)}\"" unless href
+          "<a #{href_attr}#{tag_options}>#{ERB::Util.html_escape(name || url)}</a>".html_safe
         end
       end
 
@@ -250,8 +256,9 @@ module ActionView
       # using the +link_to+ method with the <tt>:method</tt> modifier as described in
       # the +link_to+ documentation.
       #
-      # The generated form element has a class name of <tt>button_to</tt>
-      # to allow styling of the form itself and its children. You can control
+      # By default, the generated form element has a class name of <tt>button_to</tt>
+      # to allow styling of the form itself and its children. This can be changed
+      # using the <tt>:form_class</tt> modifier within +html_options+. You can control
       # the form submission and input element behavior using +html_options+.
       # This method accepts the <tt>:method</tt> and <tt>:confirm</tt> modifiers
       # described in the +link_to+ documentation. If no <tt>:method</tt> modifier
@@ -261,17 +268,20 @@ module ActionView
       # to change the HTTP verb used to submit the form.
       #
       # ==== Options
-      # The +options+ hash accepts the same options as url_for.
+      # The +options+ hash accepts the same options as +url_for+.
       #
       # There are a few special +html_options+:
-      # * <tt>:method</tt> - Symbol of HTTP verb. Supported verbs are <tt>:post</tt>, <tt>:get</tt>, 
+      # * <tt>:method</tt> - Symbol of HTTP verb. Supported verbs are <tt>:post</tt>, <tt>:get</tt>,
       #   <tt>:delete</tt> and <tt>:put</tt>. By default it will be <tt>:post</tt>.
       # * <tt>:disabled</tt> - If set to true, it will generate a disabled button.
       # * <tt>:confirm</tt> - This will use the unobtrusive JavaScript driver to
       #   prompt with the question specified. If the user accepts, the link is
       #   processed normally, otherwise no action is taken.
       # * <tt>:remote</tt> -  If set to true, will allow the Unobtrusive JavaScript drivers to control the
-      #   submit behaviour. By default this behaviour is an ajax submit.
+      #   submit behavior. By default this behavior is an ajax submit.
+      # * <tt>:form</tt> - This hash will be form attributes
+      # * <tt>:form_class</tt> - This controls the class of the form within which the submit button will
+      #   be placed
       #
       # ==== Examples
       #   <%= button_to "New", :action => "new" %>
@@ -280,6 +290,18 @@ module ActionView
       #   #    </form>"
       #
       #
+      #   <%= button_to "New", :action => "new", :form_class => "new-thing" %>
+      #   # => "<form method="post" action="/controller/new" class="new-thing">
+      #   #      <div><input value="New" type="submit" /></div>
+      #   #    </form>"
+      #
+      #
+      #   <%= button_to "Create", :action => "create", :remote => true, :form => { "data-type" => "json" } %>
+      #   # => "<form method="post" action="/images/create" class="button_to" data-remote="true" data-type="json">
+      #   #      <div><input value="Create" type="submit" /></div>
+      #   #    </form>"
+      #
+      #      
       #   <%= button_to "Delete Image", { :action => "delete", :id => @image.id },
       #             :confirm => "Are you sure?", :method => :delete %>
       #   # => "<form method="post" action="/images/delete/1" class="button_to">
@@ -309,9 +331,11 @@ module ActionView
         end
 
         form_method = method.to_s == 'get' ? 'get' : 'post'
-
+        form_options = html_options.delete('form') || {}
+        form_options[:class] ||= html_options.delete('form_class') || 'button_to'
+        
         remote = html_options.delete('remote')
-
+        
         request_token_tag = ''
         if form_method == 'post' && protect_against_forgery?
           request_token_tag = tag(:input, :type => "hidden", :name => request_forgery_protection_token.to_s, :value => form_authenticity_token)
@@ -324,8 +348,10 @@ module ActionView
 
         html_options.merge!("type" => "submit", "value" => name)
 
-        ("<form method=\"#{form_method}\" action=\"#{html_escape(url)}\" #{"data-remote=\"true\"" if remote} class=\"button_to\"><div>" +
-          method_tag + tag("input", html_options) + request_token_tag + "</div></form>").html_safe
+        form_options.merge!(:method => form_method, :action => url)
+        form_options.merge!("data-remote" => "true") if remote
+        
+        "#{tag(:form, form_options, true)}<div>#{method_tag}#{tag("input", html_options)}#{request_token_tag}</div></form>".html_safe
       end
 
 
@@ -404,7 +430,7 @@ module ActionView
       end
 
       # Creates a link tag of the given +name+ using a URL created by the set of
-      # +options+ if +condition+ is true, in which case only the name is
+      # +options+ if +condition+ is true, otherwise only the name is
       # returned. To specialize the default behavior, you can pass a block that
       # accepts the name or the full argument list for +link_to_unless+ (see the examples
       # in +link_to_unless+).
@@ -470,45 +496,41 @@ module ActionView
       #            :subject => "This is an example email"
       #   # => <a href="mailto:me@domain.com?cc=ccaddress@domain.com&subject=This%20is%20an%20example%20email">My email</a>
       def mail_to(email_address, name = nil, html_options = {})
-        email_address = html_escape(email_address)
+        email_address = ERB::Util.html_escape(email_address)
 
         html_options = html_options.stringify_keys
         encode = html_options.delete("encode").to_s
-        cc, bcc, subject, body = html_options.delete("cc"), html_options.delete("bcc"), html_options.delete("subject"), html_options.delete("body")
 
-        extras = []
-        extras << "cc=#{Rack::Utils.escape(cc).gsub("+", "%20")}" unless cc.nil?
-        extras << "bcc=#{Rack::Utils.escape(bcc).gsub("+", "%20")}" unless bcc.nil?
-        extras << "body=#{Rack::Utils.escape(body).gsub("+", "%20")}" unless body.nil?
-        extras << "subject=#{Rack::Utils.escape(subject).gsub("+", "%20")}" unless subject.nil?
-        extras = extras.empty? ? '' : '?' + html_escape(extras.join('&'))
+        extras = %w{ cc bcc body subject }.map { |item|
+          option = html_options.delete(item) || next
+          "#{item}=#{Rack::Utils.escape(option).gsub("+", "%20")}"
+        }.compact
+        extras = extras.empty? ? '' : '?' + ERB::Util.html_escape(extras.join('&'))
 
-        email_address_obfuscated = email_address.dup
-        email_address_obfuscated.gsub!(/@/, html_options.delete("replace_at")) if html_options.has_key?("replace_at")
-        email_address_obfuscated.gsub!(/\./, html_options.delete("replace_dot")) if html_options.has_key?("replace_dot")
-
-        string = ''
-
-        if encode == "javascript"
+        email_address_obfuscated = email_address.to_str
+        email_address_obfuscated.gsub!(/@/, html_options.delete("replace_at")) if html_options.key?("replace_at")
+        email_address_obfuscated.gsub!(/\./, html_options.delete("replace_dot")) if html_options.key?("replace_dot")
+        case encode
+        when "javascript"
+          string = ''
           html   = content_tag("a", name || email_address_obfuscated.html_safe, html_options.merge("href" => "mailto:#{email_address}#{extras}".html_safe))
-          html   = escape_javascript(html)
+          html   = escape_javascript(html.to_str)
           "document.write('#{html}');".each_byte do |c|
             string << sprintf("%%%x", c)
           end
           "<script type=\"#{Mime::JS}\">eval(decodeURIComponent('#{string}'))</script>".html_safe
-        elsif encode == "hex"
-          email_address_encoded = ''
-          email_address_obfuscated.each_byte do |c|
-            email_address_encoded << sprintf("&#%d;", c)
-          end
+        when "hex"
+          email_address_encoded = email_address_obfuscated.unpack('C*').map {|c|
+            sprintf("&#%d;", c)
+          }.join
 
-          protocol = 'mailto:'
-          protocol.each_byte { |c| string << sprintf("&#%d;", c) }
-
-          email_address.each_byte do |c|
+          string = 'mailto:'.unpack('C*').map { |c|
+            sprintf("&#%d;", c)
+          }.join + email_address.unpack('C*').map { |c|
             char = c.chr
-            string << (char =~ /\w/ ? sprintf("%%%x", c) : char)
-          end
+            char =~ /\w/ ? sprintf("%%%x", c) : char
+          }.join
+
           content_tag "a", name || email_address_encoded.html_safe, html_options.merge("href" => "#{string}#{extras}".html_safe)
         else
           content_tag "a", name || email_address_obfuscated.html_safe, html_options.merge("href" => "mailto:#{email_address}#{extras}".html_safe)
@@ -543,10 +565,10 @@ module ActionView
       #   current_page?(:controller => 'shop', :action => 'checkout')
       #   # => true
       #
-      #   current_page?(:controller => 'shop', :action => 'checkout', :order => 'desc', :page=>'1')
+      #   current_page?(:controller => 'shop', :action => 'checkout', :order => 'desc', :page => '1')
       #   # => true
       #
-      #   current_page?(:controller => 'shop', :action => 'checkout', :order => 'desc', :page=>'2')
+      #   current_page?(:controller => 'shop', :action => 'checkout', :order => 'desc', :page => '2')
       #   # => false
       #
       #   current_page?(:controller => 'shop', :action => 'checkout', :order => 'desc')
@@ -584,40 +606,33 @@ module ActionView
 
       private
         def convert_options_to_data_attributes(options, html_options)
-          html_options = {} if html_options.nil?
-          html_options = html_options.stringify_keys
+          if html_options.nil?
+            link_to_remote_options?(options) ? {'data-remote' => 'true'} : {}
+          else
+            html_options = html_options.stringify_keys
+            html_options['data-remote'] = 'true' if link_to_remote_options?(options) || link_to_remote_options?(html_options)
 
-          if (options.is_a?(Hash) && options.key?('remote') && options.delete('remote')) || (html_options.is_a?(Hash) && html_options.key?('remote') && html_options.delete('remote'))
-            html_options['data-remote'] = 'true'
+            disable_with = html_options.delete("disable_with")
+            confirm = html_options.delete('confirm')
+            method  = html_options.delete('method')
+
+            html_options["data-disable-with"] = disable_with if disable_with
+            html_options["data-confirm"] = confirm if confirm
+            add_method_to_attributes!(html_options, method)   if method
+
+            html_options
           end
-
-          disable_with = html_options.delete("disable_with")
-          confirm = html_options.delete("confirm")
-
-          if html_options.key?("popup")
-            ActiveSupport::Deprecation.warn(":popup has been deprecated", caller)
-          end
-
-          method, href = html_options.delete("method"), html_options['href']
-
-          add_disable_with_to_attributes!(html_options, disable_with) if disable_with
-          add_confirm_to_attributes!(html_options, confirm) if confirm
-          add_method_to_attributes!(html_options, method)   if method
-
-          html_options
         end
 
-        def add_confirm_to_attributes!(html_options, confirm)
-          html_options["data-confirm"] = confirm if confirm
-        end
-
-        def add_disable_with_to_attributes!(html_options, disable_with)
-          html_options["data-disable-with"] = disable_with if disable_with
+        def link_to_remote_options?(options)
+          options.is_a?(Hash) && options.key?('remote') && options.delete('remote')
         end
 
         def add_method_to_attributes!(html_options, method)
-          html_options["rel"] = "nofollow" if method && method.to_s.downcase != "get"
-          html_options["data-method"] = method if method
+          if method && method.to_s.downcase != "get"
+            html_options["rel"] = "#{html_options["rel"].to_s} nofollow".split(" ").uniq.join(" ")
+          end
+          html_options["data-method"] = method
         end
 
         def options_for_javascript(options)
